==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webdav/inc/datastore.php <==
<?php
//Zkladn tda pro komunikaci s Merak GroupWare
//Base clase for communication with Merak GroupWare

include_once 'gw.php';

$datastore_types=array('VEVENT'=>'E','VJOURNAL'=>'J','VTODO'=>'T','VNOTE'=>'N','VCARD'=>'C');
$content_types=array('C'=>'text/x-vcard','E'=>'text/calendar','J'=>'text/calendar','N'=>'text/x-vnote','T'=>'text/calendar');
$collection_types=array('C'=>'vcard-collection','E'=>'vevent-collection','J'=>'vjournal-collection','N'=>'vnote-collection','T'=>'vtodo-collection');

class datastore
{
  var $_gw;
  var $_gsid;
  var $_folders;
  var $_fsid;
  var $_folder_tree;
  var $_root_folders;
  
  var $_object_type;
  var $_depth;
  var $_julian_range;
  var $_expand;
  var $_get_folders;
  var $_get_data;
  var $_get_etag;
  
  function datastore()
  {
    $this->_gw=new MerakGWAPI();
    $this->_fsid=array();
  }

//************************************************************
  
  function set_object_type($comp_filter)
  {
    global $datastore_types;
    
    if (count($comp_filter))
      $this->_object_type=$datastore_types[$comp_filter['name']];
    else
      $this->_object_type=false; 
  }

//************************************************************
  
  function set_depth($depth)
  {
    $this->_depth=$depth;
  }

//************************************************************
  
  function set_julian_range($time_range)
  {
    if (count($time_range))
    {
      foreach($time_range as $key=>$value)
      {
        $year=substr($value,0,4);
        $month=substr($value,4,2);
        $day=substr($value,6,2);
        $julian[$key]=gregoriantojd($month,$day,$year);      
      }
      $this->_julian_range=$julian['start'].'-'.$julian['end'];
    }
    else
      $this->_julian_range=false;
  }

//************************************************************
  
  function set_expand($expand)
  {
    $this->_expand=$expand;
  }

//************************************************************
  
  function format_folder($destination)
  {
    // Remove protocol
    if ($p=strpos($destination, '://')) 
      $destination=substr($destination, $p+3, strlen($destination));

    // Leave only the folder structure - remove server name if present
    $p=strpos($destination, '/');
    $destination=substr($destination, $p+1, strlen($destination));

    // Replace / with \
    return(str_replace('/', '\\', $destination));
  }

//************************************************************

  function set_folder($get_folders)
  {
    $this->_get_folders=$get_folders;
  }

//************************************************************

  function set_data($get_data)
  {
    $this->_get_data=$get_data;
  }

//************************************************************

  function set_etag($get_etag)
  {
    $this->_get_etag=$get_etag;
  }

//************************************************************
  
  function parse_result_line($text,$first,$second,$third='',$value='')
  {
    if(!$text)
      return array();
    $lines=explode("\r\n",$text);

    $fields=explode('&',strtoupper(trim($lines[0])));
    foreach($fields as $key=>$field)
    {
      if ($field==$first)
        $id=$key;
      else
        if ($field==$second)
          $counter=$key;

      if ($third && $field==$third)
        $only=$key;
		}
		
    unset($lines[0]);

    foreach($lines as $line)
    {
      $line=trim($line);
      if (!$line)
        continue;

			$part=explode("&",$line);
      if (($only && $value) && $part[$only]!=$value)
        continue;
        
      $result[rawurldecode($part[$id])]=$part[$counter];
    }
    return $result;
  }

//************************************************************
  
  function folder_tree()
  {
    $this->_folder_tree=array();
    foreach($this->_folders as $folder=>$type)
    {
       $p=str_replace('\\',"']['folders']['",$folder);
       if ($p==$folder)
         $this->_root_folders[]=$folder;
 
      $folder_str.='$'."this->_folder_tree['".$p."']['name']='".str_replace('\\','\\\\', $folder)."';";
    }   
    eval($folder_str);
  }

//************************************************************

  function getfolders()
  {
    $text=$this->_gw->FunctionCall('GetFolderList',$this->_gsid);
    $this->_folders=$this->parse_result_line($text,'FDR_ID','FDRTYPE');
    $this->folder_tree();
  }
  
//************************************************************
  
  function login($username,$password,$email)
  {
    $this->_gw->user=$username;
    $this->_gw->pass=$password;

    if ($this->_gw->Login())
    {
      if ($email)
			{
				$group=$this->_gw->GetGroups($email);
				$this->_gsid=$this->_gw->OpenGroup($group[0]['GRP_ID']);
			}
			else
				$this->_gsid=$this->_gw->OpenGroup("*");

      $this->getfolders();
      return true;
    }
    else
      return false;
  }

//************************************************************
  
  function user_logged_on()
  {
    return $this->_gw->IsConnected();
  }

//************************************************************
  
  function open_folder($folder)  
  {
    if ($object_type=$this->_folders[$folder])
    {
      if (!$this->_fsid[$folder])
        $this->_fsid[$folder]=$this->_gw->OpenFolder($this->_gsid,$folder);
    
      return $this->_fsid[$folder];
    }
    else
      return false;  
  }

//************************************************************

	function mkcalendar($folder,$body)
  {
    // Find folder type
    if (strpos($body,'VCARD'))
			$type='C';
		else
			if (strpos($body,'VEVENT'))
				$type='E';
			else
    		if (strpos($body,'VJOURNAL'))
					$type='J';
				else
    			if (strpos($body,'VNOTE'))
						$type='N';
					else
    				if (strpos($body,'VTODO'))
							$type='T';

    // Parent folder type
    if (!$type)
    {
      // Get parent folder
      $parent=substr($folder,0,strrpos($folder,'\\'));

      // Get parent object type
      $object_type=$this->_folders[$parent];

      // If known set type
      if ($object_type)
				$type=$object_type;
    }

    // Default folder type
    if (!$type)
			$type='E';

    // Add folder
    $result=$this->_gw->FunctionCall('AddFolder',$this->_gsid,$folder,$type);
    $this->getfolders();

    if ($result)
      $status='201 Created';
    else
      $status='403 Forbidden';

    return $status;
  }

//************************************************************
  
  function get_item($folder,$local_id)
  {
    global $content_types;    

    $object_type=$this->_folders[$folder];
    
    if ($object_type && $local_id)
    {
      $this->open_folder($folder);
      
      switch($object_type)      
      {
        case 'C':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ITM_ID='".$local_id."'"),'ITM_ID','ITM_EDITCOUNTER');
          
          if (count($list))
          {
            $edit_counter=intval($list[$local_id]);
            $item=$this->_gw->FunctionCall('GetvCard',$this->_fsid[$folder],$local_id,$_SERVER['HTTP_ACCEPT']);
          }
          break;
          
        case 'E':
        case 'J':
        case 'N':
        case 'T':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EVN_ID='".$local_id."'"),'EVN_ID','EVN_EDITCOUNTER');
          
          if (count($list))
          {
            $edit_counter=intval($list[$local_id]);
            $item=$this->_gw->FunctionCall('GetvCalendar',$this->_fsid[$folder],$local_id,$_SERVER['HTTP_ACCEPT']);
          }  
      }
      
      if ($item)
      {
        $result['status']='200 OK';
        $result['item']=$item;
        $result['edit_counter']=$edit_counter;
        $result['content_type']=$content_types[$object_type];
      }
      else
        $result['status']='404 Not Found';
    }
    else 
      $result['status']='409 Conflict';
    
    return $result;        
  }

//************************************************************

  function get_items($folder,$local_id,&$result,$rec=0)
  {
    if (!$folder)
    {
      foreach($this->_root_folders as $folder)
        $this->get_items($folder,$local_id,$result,0);
      return;
    }

    $object_type=$this->_folders[$folder];

    // Get items
    $items=($this->_get_etag ||(!$this->_get_folders));

    if (($this->_object_type && $object_type==$this->_object_type) || (!$this->_object_type && strpos(' CEJTN',$object_type)))
    {
      // Return folder list
      if ($this->_get_folders)
      {
        $result[$folder]['type']=$object_type;
      }

      // Return item list
      if ($items)
      {
        $this->open_folder($folder);

        if ($object_type=='C')
        {
          $id_query = isset($local_id) ? " AND ITM_ID='".$local_id."'" : "";

					$result_line=$this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ItmClass='".$object_type."'".$id_query,"Itm_EditCounter");
          $list=$this->parse_result_line($result_line,'ITM_ID','ITM_EDITCOUNTER');

          $functionname='GetVCard';
        }
        else
        {
          $id_query = isset($local_id) ? " AND EVN_ID='".$local_id."'" : "";

          if ($this->_julian_range && $object_type=='E')
					{
						$result_line=$this->_gw->FunctionCall('GetAllIntervalEvents',$this->_fsid[$folder],$this->_julian_range,"EvnClass='".$object_type."'".$id_query,"Evn_EditCounter,EvnClass");
						$list=$this->parse_result_line($result_line,'EVN_ID','EVN_EDITCOUNTER','EVNCLASS','E');
					}
					else
					{
						$result_line=$this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EvnClass='".$object_type."'".$id_query,"Evn_EditCounter");
            $list=$this->parse_result_line($result_line,'EVN_ID','EVN_EDITCOUNTER');
          }

          $functionname='GetVCalendar';
        }

        if (count($list))
          foreach($list as $key=>$value)
            if ($this->_expand || !isset($result[$key]))
            {
              $result[$key]['folder']=$folder;
              $result[$key]['counter']=$value;
              if ($this->_get_data)
                $result[$key]['data']=$this->_gw->FunctionCall($functionname,$this->_fsid[$folder],$key);
              else
                $result[$key]['data']='*';
            }
      }
    }

    if ($this->_depth=='infinity' || $this->_depth>$rec)
    {
      $p=str_replace("\\","']['folders']['",$folder);
      $folder_str='$'."this->_folder_tree['".$p."']['folders']";
      eval('$'.'sub_folders='.$folder_str.';');

      if (count($sub_folders))
        foreach($sub_folders as $sub_folder)
          $this->get_items($sub_folder['name'],$local_id,$result,$rec+1);
    }
  }

//************************************************************
  
  function put_item($folder,$local_id,$item,$verify,$if_match)
  {
    if ($folder)
    	$object_type=$this->_folders[$folder];

		//Uren default folderu
    if ((!$folder) || (!$object_type))
      if (strpos($item,'BEGIN:VCARD'))
				$default_folder='Contacts';
			else
      	if (strpos($item,'BEGIN:VEVENT'))
					$default_folder='Events';
				else
      		if (strpos($item,'BEGIN:VJOURNAL'))
						$default_folder='Journal';
					else
      			if (strpos($item,'BEGIN:VNOTE'))
							$default_folder='Notes';
						else
      				if (strpos($item,'BEGIN:VTODO'))
								$default_folder='Tasks';

		if ($default_folder)
		  $folder=$folder?$folder.'\\'.$default_folder:$default_folder;

    //Znme folder, kde budeme poloku ukldat i aktualizovat?
    if ($object_type=$this->_folders[$folder])
    {            
      $this->open_folder($folder);
      
      switch($object_type)
      {
        case 'C':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ITM_ID='".$local_id."'"),'ITM_ID','ITM_EDITCOUNTER');
          
          if (count($list))
          {
            if (!isset($if_match) || $if_match==$list[$local_id])
              $replace=$this->_gw->FunctionCall('AddvCard',$this->_fsid[$folder],$item,$local_id,$_SERVER['CONTENT_TYPE']);
          }
          else        
            $result_id=$this->_gw->FunctionCall('AddvCard',$this->_fsid[$folder],$item,';'.$local_id,$_SERVER['CONTENT_TYPE'].$verify);
          break;
    
        case 'E':
        case 'J':
        case 'N':
        case 'T':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EVN_ID='".$local_id."'"),'EVN_ID','EVN_EDITCOUNTER');
          
          if (count($list))
          {
            if (!isset($if_match) || $if_match==$list[$local_id])
              $replace=$this->_gw->FunctionCall('AddvCalendar',$this->_fsid[$folder],$item,$local_id,$_SERVER['CONTENT_TYPE']);
          }
          else      
            $result_id=$this->_gw->FunctionCall('AddvCalendar',$this->_fsid[$folder],$item,';'.$local_id,$_SERVER['CONTENT_TYPE'].$verify);
      }
      
      $result['folder']=$folder;
      
      if ($result_id)
      {
        $result['local_id']=$result_id;
        $result['status']='201 Created';
      }
      else
        if ($replace)
          $result['status']='204 No Content';
        else
          $result['status']='412 Precondition Failed';     
    }
    else
      $result['status']='409 Conflict';
   
    return $result;  
  }

//************************************************************

  function delete_item($folder,$local_id,$if_match)
  {
    //Existuje vybran folder?
    if ($object_type=$this->_folders[$folder])
    {
      $this->open_folder($folder);

      //Item or collection
      if ($local_id)
      {
        switch($object_type)
        {
          case 'C':
            $list=$this->parse_result_line($this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ITM_ID='".$local_id."'"),'ITM_ID','ITM_EDITCOUNTER');

            if (count($list))
              if (!isset($if_match) || $if_match==$list[$local_id])
              {
                $result=$this->_gw->FunctionCall('DeleteContact',$this->_fsid[$folder],$local_id);
                $match=true;
              }
            break;

          case 'E':
          case 'J':
          case 'N':
          case 'T':
            $list=$this->parse_result_line($this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EVN_ID='".$local_id."'"),'EVN_ID','EVN_EDITCOUNTER');

            if (count($list))
              if (!isset($if_match) || $if_match==$list[$local_id])
              {
                $result=$this->_gw->FunctionCall('DeleteEvent',$this->_fsid[$folder],$local_id);
                $match=true;
              }
        }
      }
      else
      {
        $result=$this->_gw->FunctionCall('DeleteFolder',$this->_gsid,$folder);
        $match=true;
				$this->getfolders();
      }

      if ($result)
        $status='204 No Content';
      else
        if ($match)
          $status='404 Not Found';
        else
          $status='412 Precondition Failed';
    }
    else
      $status='409 Conflict';

    return $status;
  }

//************************************************************

  function move_item($folder,$local_id,$newfolder)
  {
    $object_type=$this->_folders[$folder];

    $destination=$this->format_folder($newfolder);

    //Existuje vybran folder?
    if ($object_type)
    {
      $this->open_folder($folder);

      //Item or collection
      if ($local_id)
      {
        switch($object_type)
        {
          case 'C':
            $result=$this->_gw->FunctionCall('AddContactInfo',$this->_fsid[$folder],"ItmFolder='".$destination."'",$local_id);
            break;

          case 'E':
          case 'J':
          case 'N':
          case 'T':
            $result=$this->_gw->FunctionCall('AddEventInfo',$this->_fsid[$folder],"EvnFolder='".$destination."'",$local_id);
        }
      }
      else
      {
        $result=$this->_gw->FunctionCall('RenameFolder',$this->_gsid,$folder,$destination);
        $this->getfolders();
    }

      if ($result)
        $status='201 Created';
      else
        $status='403 Forbidden';
    }
    else
      $status='409 Conflict';

    return $status;
  }
}
?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webdav/inc/datastore.php <==
<?php
//Zkladn tda pro komunikaci s Merak GroupWare
//Base clase for communication with Merak GroupWare

include_once 'gw.php';

$datastore_types=array('VEVENT'=>'E','VJOURNAL'=>'J','VTODO'=>'T','VNOTE'=>'N','VCARD'=>'C');
$content_types=array('C'=>'text/x-vcard','E'=>'text/calendar','J'=>'text/calendar','N'=>'text/x-vnote','T'=>'text/calendar');
$collection_types=array('C'=>'vcard-collection','E'=>'vevent-collection','J'=>'vjournal-collection','N'=>'vnote-collection','T'=>'vtodo-collection');

class datastore
{
  var $_gw;
  var $_gsid;
  var $_folders;
  var $_fsid;
  var $_folder_tree;
  var $_root_folders;
  
  var $_depth;
  var $_object_type;
  var $_text_match;
  var $_julian_range;
  var $_expand;
  var $_get_folders;
  var $_get_data;
  var $_get_etag;
  
  function datastore()
  {
    $this->_gw=new MerakGWAPI();
    $this->_fsid=array();
  }

//************************************************************

  function set_depth($depth)
  {
    $this->_depth=$depth;
  }

//************************************************************
  
  function set_object_type($comp_filter)
  {
    global $datastore_types;
    
    if (count($comp_filter))
      $this->_object_type=$datastore_types[$comp_filter['name']];
    else
      $this->_object_type='';
  }

//************************************************************
  
  function set_text_match($text_match)
  {
		$this->_text_match=$text_match;
	}
	
//************************************************************
  
  function set_julian_range($time_range)
  {
    if (count($time_range))
    {
      foreach($time_range as $key=>$value)
      {
        $year=substr($value,0,4);
        $month=substr($value,4,2);
        $day=substr($value,6,2);
        $julian[$key]=gregoriantojd($month,$day,$year);      
      }
      $this->_julian_range=$julian['start'].'-'.$julian['end'];
    }
    else
      $this->_julian_range=false;
  }

//************************************************************
  
  function set_expand($expand)
  {
    $this->_expand=$expand;
  }

//************************************************************
  
  function format_folder($destination)
  {
    // Remove protocol
    if ($p=strpos($destination,'://'))
      $destination=substr($destination,$p+3, strlen($destination));

    // Leave only the folder structure - remove server name if present
    $p=strpos($destination,'/');
    $destination=substr($destination,$p+1, strlen($destination));

    // Replace / with \
    return(str_replace('/','\\',$destination));
  }

//************************************************************

  function set_folder($get_folders)
  {
    $this->_get_folders=$get_folders;
  }

//************************************************************

  function set_data($get_data)
  {
    $this->_get_data=$get_data;
  }

//************************************************************

  function set_etag($get_etag)
  {
    $this->_get_etag=$get_etag;
  }

//************************************************************
  
  function parse_result_line($text,$first,$second,$third='',$value='')
  {
    if(!$text)
      return array();
    $lines=explode("\r\n",$text);

    $fields=explode('&',strtoupper(trim($lines[0])));
    foreach($fields as $key=>$field)
    {
      if ($field==$first)
        $id=$key;
      else
        if ($field==$second)
          $counter=$key;

      if ($third && $field==$third)
        $only=$key;
		}
		
    unset($lines[0]);

    foreach($lines as $line)
    {
      $line=trim($line);
      if (!$line)
        continue;

			$part=explode("&",$line);
      if (($only && $value) && $part[$only]!=$value)
        continue;
        
      $result[rawurldecode($part[$id])]=$part[$counter];
    }
    return $result;
  }

//************************************************************
  
  function folder_tree()
  {
    $this->_folder_tree=array();
    foreach($this->_folders as $folder=>$type)
    {
       $p=str_replace('\\',"']['folders']['",$folder);
       if ($p==$folder)
         $this->_root_folders[]=$folder;
 
      $folder_str.='$'."this->_folder_tree['".$p."']['name']='".str_replace('\\','\\\\', $folder)."';";
    }   
    eval($folder_str);
  }

//************************************************************

  function getfolders()
  {
    $text=$this->_gw->FunctionCall('GetFolderList',$this->_gsid);
    $this->_folders=$this->parse_result_line($text,'FDR_ID','FDRTYPE');
    $this->folder_tree();
  }
  
//************************************************************
  
  function login($username,$password,$email)
  {
    $this->_gw->user=$username;
    $this->_gw->pass=$password;

    if ($this->_gw->Login())
    {
      if ($email && ($email!=$username))
			{
				$group=$this->_gw->GetGroups($email);
				$this->_gsid=$this->_gw->OpenGroup($group[0]['GRP_ID']);
			}
			else
				$this->_gsid=$this->_gw->OpenGroup("*");

      $this->getfolders();
      return true;
    }
    else
      return false;
  }

//************************************************************
  
  function user_logged_on()
  {
    return $this->_gw->IsConnected();
  }

//************************************************************
  
  function open_folder($folder)  
  {
    if ($object_type=$this->_folders[$folder])
    {
      if (!$this->_fsid[$folder])
        $this->_fsid[$folder]=$this->_gw->OpenFolder($this->_gsid,$folder);
    
      return $this->_fsid[$folder];
    }
    else
      return false;  
  }

//************************************************************

	function mkcalendar($folder,$body)
  {
    // Find folder type
    if (strpos($body,'VCARD'))
			$type='C';
		else
			if (strpos($body,'VEVENT'))
				$type='E';
			else
    		if (strpos($body,'VJOURNAL'))
					$type='J';
				else
    			if (strpos($body,'VNOTE'))
						$type='N';
					else
    				if (strpos($body,'VTODO'))
							$type='T';

    // Parent folder type
    if (!$type)
    {
      // Get parent folder
      $parent=substr($folder,0,strrpos($folder,'\\'));

      // Get parent object type
      $object_type=$this->_folders[$parent];

      // If known set type
      if ($object_type)
				$type=$object_type;
    }

    // Default folder type
    if (!$type)
			$type='E';

    // Add folder
    $result=$this->_gw->FunctionCall('AddFolder',$this->_gsid,$folder,$type);
    $this->getfolders();

    if ($result)
      $status='201 Created';
    else
      $status='403 Forbidden';

    return $status;
  }

//************************************************************
  
  function get_item($folder,$local_id)
  {
    global $content_types;    

    $object_type=$this->_folders[$folder];
    
    if ($object_type && $local_id)
    {
      $this->open_folder($folder);
      
      switch($object_type)      
      {
        case 'C':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ITM_ID='".$local_id."'",'Itm_EditCounter'),'ITM_ID','ITM_EDITCOUNTER');
          
          if (count($list))
          {
            $edit_counter=intval($list[$local_id]);
            $item=$this->_gw->FunctionCall('GetvCard',$this->_fsid[$folder],$local_id,$_SERVER['HTTP_ACCEPT']);
          }
          break;
          
        case 'E':
        case 'J':
        case 'N':
        case 'T':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EVN_ID='".$local_id."'",'Evn_EditCounter'),'EVN_ID','EVN_EDITCOUNTER');
          
          if (count($list))
          {
            $edit_counter=intval($list[$local_id]);
            $item=$this->_gw->FunctionCall('GetvCalendar',$this->_fsid[$folder],$local_id,$_SERVER['HTTP_ACCEPT']);
          }  
      }
      
      if ($item)
      {
        $result['status']='200 OK';
        $result['data']=$item;
        $result['counter']=$edit_counter;
        $result['type']=$content_types[$object_type];
      }
      else
        $result['status']='404 Not Found';
    }
    else 
      $result['status']='409 Conflict';
    
    return $result;        
  }

//************************************************************

  function get_items($folder,$local_id,&$result,$rec=0)
  {
		global $content_types,$collection_types;

		if ((!$this->_get_etag) && (!$this->_get_folders) && (!$this->_get_data))
			return;

		if (!$folder)
    {
			if ($this->_depth)
				foreach($this->_root_folders as $folder)
        	$this->get_items($folder,$local_id,$result,0);
      return;
    }

    $object_type=$this->_folders[$folder];

    if (($this->_object_type && ($object_type==$this->_object_type)) || (!$this->_object_type && strpos(' CEJTN',$object_type)))
    {
      if ($this->_get_folders)
      {
				$result[$folder]['type']=$object_type;
				$result[$folder]['collection']=$collection_types[$object_type];
			}

      if ($this->_get_etag ||$this->_get_data)
      {
				$this->open_folder($folder);

        if ($object_type=='C')
        {
          $id_query=(isset($local_id)?(" AND ITM_ID='".$local_id."'"):'');

					$result_line=$this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ItmClass='".$object_type."'".$id_query,'Itm_EditCounter');
          $list=$this->parse_result_line($result_line,'ITM_ID','ITM_EDITCOUNTER');

          $functionname='GetVCard';
        }
        else
        {
          $id_query=(isset($local_id)?(" AND EVN_ID='".$local_id."'"):'');

          if ($this->_julian_range && $object_type=='E')
					{
						$result_line=$this->_gw->FunctionCall('GetAllIntervalEvents',$this->_fsid[$folder],$this->_julian_range,"EvnClass='".$object_type."'".$id_query,'Evn_EditCounter,EvnClass');
						$list=$this->parse_result_line($result_line,'EVN_ID','EVN_EDITCOUNTER','EVNCLASS','E');
					}
					else
					{
						$result_line=$this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EvnClass='".$object_type."'".$id_query,'Evn_EditCounter');
            $list=$this->parse_result_line($result_line,'EVN_ID','EVN_EDITCOUNTER');
          }

          $functionname='GetVCalendar';
        }

        if (count($list))
          foreach($list as $key=>$value)
            if ((!$this->_text_match || (strpos($key,$this->_text_match)===0)) && ($this->_expand || !isset($result[$key])))
            {
              $result[$key]['folder']=$folder;
              $result[$key]['counter']=$value;
              $result[$key]['type']=$content_types[$object_type];
              if ($this->_get_data)
                $result[$key]['data']=$this->_gw->FunctionCall($functionname,$this->_fsid[$folder],$key);
              else
                $result[$key]['data']='*';
            }
      }
    }

    if ($this->_depth=='infinity' || $this->_depth>$rec)
    {
      $p=str_replace("\\","']['folders']['",$folder);
      $folder_str='$'."this->_folder_tree['".$p."']['folders']";
      eval('$'.'sub_folders='.$folder_str.';');

      if (count($sub_folders))
        foreach($sub_folders as $sub_folder)
          $this->get_items($sub_folder['name'],$local_id,$result,$rec+1);
    }
  }

//************************************************************
  
  function put_item($folder,$local_id,$item,$verify,$if_match)
  {
    if ($folder)
    	$object_type=$this->_folders[$folder];

		//Uren default folderu
    if ((!$folder) || (!$object_type))
      if (strpos($item,'BEGIN:VCARD'))
				$default_folder='Contacts';
			else
      	if (strpos($item,'BEGIN:VEVENT'))
					$default_folder='Events';
				else
      		if (strpos($item,'BEGIN:VJOURNAL'))
						$default_folder='Journal';
					else
      			if (strpos($item,'BEGIN:VNOTE'))
							$default_folder='Notes';
						else
      				if (strpos($item,'BEGIN:VTODO'))
								$default_folder='Tasks';

		if ($default_folder)
		  $folder=$folder?$folder.'\\'.$default_folder:$default_folder;

    //Znme folder, kde budeme poloku ukldat i aktualizovat?
    if ($object_type=$this->_folders[$folder])
    {            
      $this->open_folder($folder);
      
			switch($object_type)
      {
        case 'C':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ITM_ID='".$local_id."'",'Itm_EditCounter'),'ITM_ID','ITM_EDITCOUNTER');
          
          if (count($list))
          {
            if (!isset($if_match) || $if_match==$list[$local_id])
              $replace=$this->_gw->FunctionCall('AddvCard',$this->_fsid[$folder],$item,$local_id,$_SERVER['CONTENT_TYPE']);
          }
          else        
            $result_id=$this->_gw->FunctionCall('AddvCard',$this->_fsid[$folder],$item,';ID='.urlencode($local_id),$_SERVER['CONTENT_TYPE'].$verify);
          break;
    
        case 'E':
        case 'J':
        case 'N':
        case 'T':
          $list=$this->parse_result_line($this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EVN_ID='".$local_id."'",'Evn_EditCounter'),'EVN_ID','EVN_EDITCOUNTER');

          if (count($list))
          {
						if (!isset($if_match) || $if_match==$list[$local_id])
              $replace=$this->_gw->FunctionCall('AddvCalendar',$this->_fsid[$folder],$item,$local_id,$_SERVER['CONTENT_TYPE']);
					}
          else
            $result_id=$this->_gw->FunctionCall('AddvCalendar',$this->_fsid[$folder],$item,';ID='.urlencode($local_id),$_SERVER['CONTENT_TYPE'].$verify);
			}

      $result['folder']=$folder;
      
      if ($result_id)
      {
        $result['etag']=$result_id.':0';
        $result['status']='201 Created';
      }
      else
        if ($replace)
        {
          $result['etag']=$local_id.':'.(++$list[$local_id]);
          $result['status']='204 No Content';
				}
				else
          $result['status']='412 Precondition Failed';     
    }
    else
      $result['status']='409 Conflict';
   
    return $result;  
  }

//************************************************************

  function delete_item($folder,$local_id,$if_match)
  {
    //Existuje vybran folder?
    if ($object_type=$this->_folders[$folder])
    {
      $this->open_folder($folder);

      //Item or collection
      if ($local_id)
      {
        switch($object_type)
        {
          case 'C':
            $list=$this->parse_result_line($this->_gw->FunctionCall('GetContactList',$this->_fsid[$folder],"ITM_ID='".$local_id."'",'Itm_EditCounter'),'ITM_ID','ITM_EDITCOUNTER');

            if (count($list))
              if (!isset($if_match) || $if_match==$list[$local_id])
              {
                $result=$this->_gw->FunctionCall('DeleteContact',$this->_fsid[$folder],$local_id);
                $match=true;
              }
            break;

          case 'E':
          case 'J':
          case 'N':
          case 'T':
            $list=$this->parse_result_line($this->_gw->FunctionCall('GetEventList',$this->_fsid[$folder],"EVN_ID='".$local_id."'",'Evn_EditCounter'),'EVN_ID','EVN_EDITCOUNTER');

            if (count($list))
              if (!isset($if_match) || $if_match==$list[$local_id])
              {
                $result=$this->_gw->FunctionCall('DeleteEvent',$this->_fsid[$folder],$local_id);
                $match=true;
              }
        }
      }
      else
      {
        $result=$this->_gw->FunctionCall('DeleteFolder',$this->_gsid,$folder);
        $match=true;
				$this->getfolders();
      }

      if ($result)
        $status='204 No Content';
      else
        if ($match)
          $status='404 Not Found';
        else
          //$status='412 Precondition Failed';
          $status='404 Not Found';
    }
    else
      $status='409 Conflict';

    return $status;
  }

//************************************************************

  function move_item($folder,$local_id,$newfolder)
  {
    $object_type=$this->_folders[$folder];

    $destination=$this->format_folder($newfolder);

    //Existuje vybran folder?
    if ($object_type)
    {
      $this->open_folder($folder);

      //Item or collection
      if ($local_id)
      {
        switch($object_type)
        {
          case 'C':
            $result=$this->_gw->FunctionCall('AddContactInfo',$this->_fsid[$folder],"ItmFolder='".$destination."'",$local_id);
            break;

          case 'E':
          case 'J':
          case 'N':
          case 'T':
            $result=$this->_gw->FunctionCall('AddEventInfo',$this->_fsid[$folder],"EvnFolder='".$destination."'",$local_id);
        }
      }
      else
      {
        $result=$this->_gw->FunctionCall('RenameFolder',$this->_gsid,$folder,$destination);
        $this->getfolders();
    	}

      if ($result)
        $status='201 Created';
      else
        $status='403 Forbidden';
    }
    else
      $status='409 Conflict';

    return $status;
  }
}
?>
